package com.esignlive.guidance.forum;

import java.text.MessageFormat;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import com.silanis.esl.sdk.Document;
import com.silanis.esl.sdk.DocumentPackage;
import com.silanis.esl.sdk.EslClient;
import com.silanis.esl.sdk.PackageId;
import com.silanis.esl.sdk.Signature;
import com.silanis.esl.sdk.Signer;

enum DocumentStatus {
	NOTREQUIRED, INPROGRESS, COMPLETE, OPTIONAL
}

public class _22_10_06_signing_status_and_signing_order {
	public static final String API_KEY = "your_api_key";
	public static final String API_URL = "https://sandbox.esignlive.com/api";
	public static final PackageId packageId = new PackageId("7crLHW7-zMN6E4a5egL0K5tijKs=");

	public static void main(String[] args) {
		_22_10_06_signing_status_and_signing_order instance = new _22_10_06_signing_status_and_signing_order();

		EslClient eslClient = new EslClient(API_KEY, API_URL);
		final DocumentPackage pkg = eslClient.getPackage(packageId);

		System.out.println("package id: " + pkg.getId());

		List<Signer> signers = pkg.getSigners();
		Collections.sort(signers, new Comparator<Signer>() {
		    @Override
		    public int compare(Signer signer1, Signer signer2) {
		        return signer1.getSigningOrder() - signer2.getSigningOrder();
		    }
		});
		
		for (Signer signer : signers) {
			String format = MessageFormat.format("signer email: {0}; signing order: {1}; status: {2};", signer.getEmail(), signer.getSigningOrder(), instance.getSigningStatusBySigner(pkg, signer.getEmail()));
			System.out.println(format);
		}

		for (Signer signer : signers) {
			DocumentStatus signingStatusBySigner = instance.getSigningStatusBySigner(pkg, signer.getEmail());
			if(signingStatusBySigner == DocumentStatus.INPROGRESS) {
				String format = MessageFormat.format("pending signer email: {0}; signing order: {1};", signer.getEmail(), signer.getSigningOrder());
				System.out.println(format);
				break;
			}
		}
		
	}

	public DocumentStatus getSigningStatusByDocument(DocumentPackage pkg, String documentName) {
		DocumentStatus status = DocumentStatus.NOTREQUIRED;

		Document document = pkg.getDocument(documentName);
		for (Signer signer : pkg.getSigners()) {
			DocumentStatus curStatus = getSigningStatusBySignerAndDocument(document, signer.getEmail());
			if (curStatus == DocumentStatus.INPROGRESS) {
				status = DocumentStatus.INPROGRESS;
				break;
			} else if (curStatus == DocumentStatus.COMPLETE) {
				status = DocumentStatus.COMPLETE;
			}
		}
		return status;
	}

	public DocumentStatus getSigningStatusBySigner(DocumentPackage pkg, String signerEmail) {
		DocumentStatus status = DocumentStatus.NOTREQUIRED;

		List<Document> documents = pkg.getDocuments();
		for (Document document : documents) {
			DocumentStatus curStatus = getSigningStatusBySignerAndDocument(document, signerEmail);
			if (curStatus == DocumentStatus.INPROGRESS) {
				status = DocumentStatus.INPROGRESS;
				break;
			} else if (curStatus == DocumentStatus.COMPLETE) {
				status = DocumentStatus.COMPLETE;
			}

		}
		return status;
	}

	public DocumentStatus getSigningStatusBySignerAndDocument(Document document, String signerEmail) {
		DocumentStatus status = DocumentStatus.NOTREQUIRED;

		for (Signature signature : document.getSignatures()) {
			if (signature.getSignerEmail().equals(signerEmail)) {
				if (signature.getAccepted() != null) {
					status = DocumentStatus.COMPLETE;
				} else if (signature.isOptional()) {
					if (status == DocumentStatus.NOTREQUIRED) {
						status = DocumentStatus.OPTIONAL;
					}
				} else {
					status = DocumentStatus.INPROGRESS;
					break;
				}
			}
		}
//		if (status == DocumentStatus.OPTIONAL) {
//			status = DocumentStatus.INPROGRESS;
//		}
		return status;
	}

}